<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Catequil\Db\Connection;
use Doctrine\Common\Annotations\AnnotationReader;
use Doctrine\Common\Annotations\PsrCachedReader;
use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Exception\ORMException;
use Doctrine\ORM\Mapping\Driver\AnnotationDriver;
use Doctrine\ORM\Mapping\Driver\AttributeDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Cache\CacheItemPoolInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Component\Cache\Adapter\ArrayAdapter;

/**
 * Class EntityManagerFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class EntityManagerFactory implements FactoryInterface
{
    private ?CacheItemPoolInterface $cache = null;

    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return EntityManagerInterface
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): EntityManagerInterface
    {
        $config = $container->get('config') ?? [];
        $isDevelopment = $config['development'] ?? false;
        $config = $config['orm'] ?? [];

        /** @var Connection $connection */
        $connection = $container->get($config['connection'] ?? 'db');
        $configuration = $this->createConfiguration($config, $isDevelopment);
        $events = $connection->getEventManager();

        try {
            return EntityManager::create($connection, $configuration, $events);
        } catch (ORMException $e) {
            throw new ServiceNotCreatedException('Can\'t create the entity manager.', 0, $e);
        }
    }

    /**
     * @param array $config
     * @param bool $isDevelopment
     *
     * @return Configuration
     */
    private function createConfiguration(array $config, bool $isDevelopment = false): Configuration
    {
        $cache = $this->createCache();
        $metadataDriver = $this->createMetadataDriver($config, $isDevelopment);

        $configuration = new Configuration();
        $configuration->setMetadataCache($cache);
        $configuration->setQueryCache($cache);
        $configuration->setResultCache($cache);
        $configuration->setProxyDir($config['proxy_dir'] ?? null);
        $configuration->setProxyNamespace('Catequil\\ORM\\Proxy\\');
        $configuration->setAutoGenerateProxyClasses($isDevelopment);
        $configuration->setMetadataDriverImpl($metadataDriver);

        return $configuration;
    }

    private function createCache(): CacheItemPoolInterface
    {
        if ($this->cache === null) {
            $this->cache = new ArrayAdapter();
        }
        return $this->cache;
    }

    /**
     * @param array $config
     * @param bool $isDevelopment
     *
     * @return MappingDriver
     */
    private function createMetadataDriver(array $config, bool $isDevelopment = false): MappingDriver
    {
        $cache = $this->createCache();
        $driver = new MappingDriverChain();

        $annotationReader = new PsrCachedReader(new AnnotationReader(), $cache, $isDevelopment);

        foreach ($config['entity_paths'] ?? [] as $namespace => $paths) {
            $annotationDriver = new AnnotationDriver($annotationReader, (array) $paths);
            $driver->addDriver($annotationDriver, $namespace);

            $attributeDriver = new AttributeDriver((array) $paths);
            $driver->addDriver($attributeDriver, $namespace);
        }

        return $driver;
    }
}
