<?php
declare(strict_types=1);

namespace CatequilTest\ORM\Filter;

use Catequil\ORM\Filter\EntityFilter;
use CatequilTest\ORM\AbstractTestCase;
use Doctrine\ORM\EntityManagerInterface;
use Laminas\Filter\FilterInterface;
use Laminas\Filter\FilterPluginManager;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use RuntimeException;

/**
 * Class EntityFilterTest
 *
 * @package CatequilTest\ORM\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil
 */
final class EntityFilterTest extends AbstractTestCase
{
    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testIsRegistered()
    {
        $container = $this->getContainer();
        /** @var FilterPluginManager $filters */
        $filters = $container->get(FilterPluginManager::class);

        self::assertTrue($filters->has(EntityFilter::class));
        self::assertInstanceOf(FilterInterface::class, $filters->get(EntityFilter::class));
    }

    public function testWillThrowExceptionIfEntityClassNotSet()
    {
        $em = $this->createMock(EntityManagerInterface::class);
        $filter = new EntityFilter($em);

        self::expectException(RuntimeException::class);
        self::assertNull($filter->filter(1));
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testWillThrowExceptionIfEntityClassDoesntExist()
    {
        $container = $this->getContainer();
        /** @var FilterPluginManager $filters */
        $filters = $container->get(FilterPluginManager::class);

        $filter = $filters->get(EntityFilter::class, [
            'entity_class' => '\DoesntExist',
        ]);

        self::expectException(RuntimeException::class);
        self::assertNull($filter->filter(1));
    }

    public function testWillThrowExceptionIfEntityNotFound()
    {
        $em = $this->createMock(EntityManagerInterface::class);
        $em->expects($this->once())
            ->method('find')
            ->with(self::class, 1)
            ->willReturn(null);

        $filter = new EntityFilter($em, [
            'entity_class' => self::class,
        ]);

        self::expectException(RuntimeException::class);
        self::assertNull($filter->filter(1));
    }

    public function testWillFilterTheEntity()
    {
        $em = $this->createMock(EntityManagerInterface::class);
        $em->expects($this->once())
            ->method('find')
            ->with(self::class, 1)
            ->willReturn($this);

        $filter = new EntityFilter($em, [
            'entity_class' => self::class,
        ]);

        self::assertSame($this, $filter->filter(1));
    }
}
